//////////////////////////////////////////////////////////////////////////////
// FakeDate\fakedate.cpp            Padre & Lord Link 06/1999
// http://www.geocities.com/yesh20     Mail: yesh20@yahoo.com
//////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <tchar.h>
#include <direct.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>


// versions
// --------
// 1.000  1999/06/18  initial
// 1.001  1999/12/31  bug fix: LogClose time now correct

static TCHAR FD_version[] = "FakeDate version 1.001, 1999/12/31";


// constants...
// ...general
const int    FD_maxargs         = 256;
const int    FD_unspecified     = -1;
const int    FD_buflen          = 4096;
const TCHAR  FD_realname[]      = "fakedate";
// ...default values for user-definable parameters
const int    FD_defaultCleanup  = 5;
const int    FD_defaultTimer    = 10;
// recongnized parameter values
const TCHAR  FD_showMaximized[] = "maximize";
const TCHAR  FD_showMinimized[] = "minimize";
const TCHAR  FD_showNormal[]    = "normal";
// ...command line options
const TCHAR  FD_debug[]         = "debug";
const TCHAR  FD_profile[]       = "profile";
const TCHAR  FD_runapp[]        = "run";
const TCHAR  FD_cmdline[]       = "cmdline";
const TCHAR  FD_date[]          = "date";
const TCHAR  FD_timer[]         = "timer";
const TCHAR  FD_stay[]          = "stay";
const TCHAR  FD_cleanup[]       = "cleanup";
const TCHAR  FD_window[]        = "window";
const TCHAR  FD_optstring[]     = "-fakedateargs";
// ...error levels
const TCHAR  FD_info[]          = "Info   ";
const TCHAR  FD_warn[]          = "Warning";
const TCHAR  FD_error[]         = "Error  ";
const TCHAR  FD_fatal[]         = "Fatal  ";
// miscellaneous
const TCHAR  FD_separator[]     = "----------------------------------------"
                                  "--------------------------------------\n";
const TCHAR  FD_mutexwarning[]  = "There's another instance of %s "
                                  "running! This will probably lead to "
                                  "conflicts regarding date manipulation. "
                                  "Do you wish to continue anyway?";
const TCHAR  FD_mutexerror[]    = "Unable to find out if there are other "
                                  "%s instances running!\nAborting program.";


// array for command line arguments
static struct {
  int argc;
  TCHAR* argv[FD_maxargs];
} FD_Args = {
  0, ""
};

// global variables
static struct {
  // user-definable parameters
  TCHAR* profile;      TCHAR* dateArg;
  TCHAR* runapp;       TCHAR* cmdline;  
  TCHAR* windowArg;    int    debug;
  int    stay;         long   timer;
  long   childCleanup; int    cmdShow;
  // process characteristics
  HANDLE runappHandle;   DWORD  runappThreadID;
  // time-specific variables
  SYSTEMTIME systmOrig;  SYSTEMTIME systmFaked;
  time_t elapsedSeconds;
  // logging-specific variables
  FILE* log;
} FD_Globals = {
  NULL, NULL, NULL, NULL, NULL,
  FD_unspecified, FD_unspecified, FD_unspecified, FD_unspecified, FALSE,
  NULL, 0,
  {0, 0, 0, 0, 0, 0, 0, 0 }, {0, 0, 0, 0, 0, 0, 0, 0 }, 0,
  NULL
};


int GetSemaphore()
{
  TCHAR message[FD_buflen];
  SECURITY_ATTRIBUTES sa = { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE };
  HANDLE handle = CreateMutex(&sa, FALSE, FD_realname);
  if (GetLastError() == ERROR_ALREADY_EXISTS) {
    sprintf(message, FD_mutexwarning, FD_realname);
    return((MessageBox(NULL, message, FD_warn, MB_YESNO |
           MB_ICONWARNING) == IDYES) ? 0 : 1);
  }
  if (handle == NULL) {
    sprintf(message, FD_mutexerror, FD_realname);
    MessageBox(NULL, message, FD_error, MB_OK | MB_ICONERROR);
    return(-1);
  }
  return(0);
}


int LogInit()
{
  int rc = 0;
  TCHAR logFilename[MAX_PATH+1];
  TCHAR drive[_MAX_DRIVE];
  TCHAR dir[_MAX_DIR];
  _splitpath(__argv[0], drive, dir, NULL, NULL);
  _stprintf(logFilename, "%c:%s%s.log", *drive, dir, FD_realname);
  // open log file
  if ((FD_Globals.log = fopen(logFilename,"wc")) == NULL) {
    FD_Globals.log = stderr;
    _ftprintf(stderr, "Unable to open file '%s' as log file, using stderr.\n",
            logFilename);
    rc = -1;
  }
  // generate starting message
  // initialize time
  GetLocalTime(&FD_Globals.systmOrig);
  _ftprintf(FD_Globals.log, "Error logging of %s started on %04d/%02d/%02d, "
            "%02d:%02d:%02d GMT\n", FD_realname, 
            FD_Globals.systmOrig.wYear, FD_Globals.systmOrig.wMonth,
            FD_Globals.systmOrig.wDay, FD_Globals.systmOrig.wHour,
            FD_Globals.systmOrig.wMinute, FD_Globals.systmOrig.wSecond);
  _ftprintf(FD_Globals.log, FD_separator);
  return(rc);
}


int Log(const TCHAR* level, TCHAR* message, ...)
{
  if (message == NULL)
    return(-1);
  // print informationals only in debug mode
  if (!_tcsicmp(level, FD_info) && FD_Globals.debug != 1)
    return(0);
  TCHAR* logMsg = new TCHAR[FD_buflen];
  va_list varArgs;
  va_start(varArgs, message);
  _vstprintf(logMsg, message, varArgs);
  // compute time
  long elapsedSeconds = FD_Globals.elapsedSeconds;
  long hrs = (long)elapsedSeconds / 3600;
  long min = ((long)elapsedSeconds - hrs*3600) / 60;
  long sec = elapsedSeconds % 60;
  // print log message
  if (_ftprintf(FD_Globals.log, "[%02ld:%02ld:%02ld] %s : %s\n",
                hrs, min, sec, level, logMsg) < 0) {
    fprintf(stderr,"Log: Write to log failed!\n");
    return(-1);
  }
  return(0);
}


void LogClose()
{
  // Generate goodbye message
  _ftprintf(FD_Globals.log, FD_separator);
  _ftprintf(FD_Globals.log, "Error logging terminated on %04d/%02d/%02d, "
            "%02d:%02d:%02d GMT",
            FD_Globals.systmOrig.wYear, FD_Globals.systmOrig.wMonth,
            FD_Globals.systmOrig.wDay, FD_Globals.systmOrig.wHour,
            FD_Globals.systmOrig.wMinute, FD_Globals.systmOrig.wSecond);
  // close log file
  if (FD_Globals.log != stderr)
    fclose(FD_Globals.log);
}


// helper function: read command line string
void String2Args(TCHAR* string)
{
  BOOL quote = FALSE;
  TCHAR* buffer = new TCHAR[_tcslen(string)];
  TCHAR* pbuf = buffer;
  TCHAR* p = string;
  TCHAR* p2 = p;
  while (*p2) {
    if ((*p2 == ' ' || *p2 == '\t') && !quote) {
      if (_tcslen(p)) {
        *pbuf = 0;
        if (FD_Args.argc < FD_maxargs) {
          FD_Args.argv[FD_Args.argc] = new char[_tcslen(buffer)+1];
          _tcscpy(FD_Args.argv[FD_Args.argc++], buffer);
        }
        pbuf = buffer;
      }
      p = p2 + 1;
    } else if (*p2 == '\"' && (p2 == p || *(p2-1) != '\\'))
      quote = !quote;
    else if (*p2 == '\"')
      --*pbuf++ = *p2;
    else
      *pbuf++ = *p2;
    p2++;
  };
  if (_tcslen(p)) {
    *pbuf = 0;
    if (FD_Args.argc < FD_maxargs) {
      FD_Args.argv[FD_Args.argc] = new char[_tcslen(buffer)+1];
      _tcscpy(FD_Args.argv[FD_Args.argc++], buffer);
    }
    pbuf = buffer;
  }
  delete[] buffer;
}


// get specified argument from command line string
TCHAR* GetArg(const TCHAR* argname)
{
  if (argname == NULL)
    return(NULL);
  for (int arg=0; arg<FD_Args.argc; arg++)
    if ((*FD_Args.argv[arg] == '-' || *FD_Args.argv[arg] == '/')
        && !_tcsicmp(argname, FD_Args.argv[arg]+1))
      return((arg < FD_Args.argc - 1)
             ? FD_Args.argv[arg+1] : strdup(""));
  return(NULL);
}


// parse the command line string
void ParseCommandLine(TCHAR* commandLine)
{
  TCHAR* p;
  String2Args(commandLine);
  if (GetArg(FD_debug) != NULL)
    FD_Globals.debug = TRUE;
  FD_Globals.profile = GetArg(FD_profile);
  FD_Globals.runapp  = GetArg(FD_runapp);
  FD_Globals.cmdline = GetArg(FD_cmdline);
  FD_Globals.dateArg = GetArg(FD_date);
  if (GetArg(FD_stay) != NULL)
    FD_Globals.stay = 1;
  if ((p = GetArg(FD_timer)) != NULL)
    FD_Globals.timer = atol(p);
  if ((p = GetArg(FD_cleanup)) != NULL)
    FD_Globals.childCleanup = atol(p);
  FD_Globals.windowArg = GetArg(FD_window);
}


// change to the directory the child application runs in
void ChangeDirToRunApp()
{
  char drive[_MAX_DRIVE];
  char dir[_MAX_DIR];
  char fname[_MAX_FNAME];
  char ext[_MAX_EXT];
  if (FD_Globals.runapp == NULL)
    return;
  _splitpath(FD_Globals.runapp, drive, dir, fname, ext);
  if (*drive != 0)
    _chdrive(*drive - 'A');
  if (*dir != 0)
    _tchdir(dir);
}


// determine location of profile
// the search order is:
// 1. the child application's directory
// 2. the directory where fakedate resides
// 3. the current working directory
BOOL GetProfileLocation()
{
  // check if profile was specified on the command line
  if (FD_Globals.profile != NULL)
    return((access(FD_Globals.profile, 0)) ? FALSE : TRUE);
  // if a child application was specified, change to the
  //  application's directory
  FD_Globals.profile = new char[MAX_PATH+1];
  if (FD_Globals.runapp != NULL) {
    ChangeDirToRunApp();
    _tgetcwd(FD_Globals.profile, MAX_PATH-13);
    _tcscat(FD_Globals.profile, "\\");
    _tcscat(FD_Globals.profile, FD_realname);
    _tcscat(FD_Globals.profile, ".ini");
    if (!access(FD_Globals.profile, 0))
      return(TRUE);
  }
  // expect profile to be in the directory where fakedate resides
  TCHAR drive[_MAX_DRIVE];
  TCHAR dir[_MAX_DIR];
  _splitpath(__argv[0], drive, dir, NULL, NULL);
  _stprintf(FD_Globals.profile, "%c:%s%s.ini", *drive, dir, FD_realname);
  if (!access(FD_Globals.profile, 0))
    return(TRUE);
  // last chance: current working directory becomes directory for profile 
  _tgetcwd(FD_Globals.profile, MAX_PATH-13);
  _tcscat(FD_Globals.profile, "\\");
  _tcscat(FD_Globals.profile, FD_realname);
  _tcscat(FD_Globals.profile, ".ini");
  return((access(FD_Globals.profile, 0)) ? FALSE : TRUE);
}


// read INI file
void ReadPrivateProfile()
{
  // try to find profile
  BOOL profileExists = GetProfileLocation();
  if (!profileExists)
    Log(FD_warn, "Unable to find profile!");
  else
    Log(FD_info, "Reading profile '%s'.", FD_Globals.profile);
  // read keys from profile...
  // ...first look for debug flag (needed to determine if the
  //    next informational should be written to log file)
  if (FD_Globals.debug == FD_unspecified) {
    TCHAR s[256] = "";
    GetPrivateProfileString(FD_realname, FD_debug, "", s,
                            256, FD_Globals.profile);
    FD_Globals.debug = (!_tcsicmp(s, "yes")) ? 1 : 0;
  }
  if (FD_Globals.runapp == NULL) {
    FD_Globals.runapp = new char[MAX_PATH+1];
    GetPrivateProfileString(FD_realname, FD_runapp, "", FD_Globals.runapp,
                            MAX_PATH+1, FD_Globals.profile);
  }
  ChangeDirToRunApp();
  if (FD_Globals.cmdline == NULL) {
    FD_Globals.cmdline = new char[256];
    GetPrivateProfileString(FD_realname, FD_cmdline, "", FD_Globals.cmdline,
                            256, FD_Globals.profile);
  }
  if (FD_Globals.dateArg == NULL) {
    FD_Globals.dateArg = new char[40];
    GetPrivateProfileString(FD_realname, FD_date, "", FD_Globals.dateArg,
                            40, FD_Globals.profile);
  }
  if (FD_Globals.stay == FD_unspecified) {
    TCHAR s[256] = "";
    GetPrivateProfileString(FD_realname, FD_stay, "", s,
                            256, FD_Globals.profile);
    FD_Globals.stay = (!_tcsicmp(s, "yes")) ? 1 : 0;
  }
  if (FD_Globals.timer == FD_unspecified)
    FD_Globals.timer = GetPrivateProfileInt(FD_realname, FD_timer,
                            0, FD_Globals.profile);
  if (FD_Globals.childCleanup == FD_unspecified)
    FD_Globals.childCleanup = GetPrivateProfileInt(FD_realname, FD_cleanup,
                            0, FD_Globals.profile);
  if (FD_Globals.windowArg == NULL) {
    FD_Globals.windowArg = new char[256];
    GetPrivateProfileString(FD_realname, FD_window, "", FD_Globals.windowArg,
                            256, FD_Globals.profile);
  }
  // set show window parameter
  if (FD_Globals.windowArg != NULL && *(FD_Globals.windowArg) != 0) {
    if (!strcmp(FD_Globals.windowArg, FD_showMaximized))
      FD_Globals.cmdShow = SW_SHOWMAXIMIZED;
    else if (!strcmp(FD_Globals.windowArg, FD_showMinimized))
      FD_Globals.cmdShow = SW_SHOWMINNOACTIVE;
    else if (!strcmp(FD_Globals.windowArg, FD_showNormal))
      FD_Globals.cmdShow = SW_SHOWNORMAL;
    else
      Log(FD_warn, "Unknown value for window appearance parameter: '%s'",
          FD_Globals.windowArg);
  }
}


// compute the faked date
BOOL ComputeFakedDate()
{
  TCHAR* p = _tcsdup(FD_Globals.dateArg);
  TCHAR* p2, *p3;
  if ((p2 = _tcschr(p, '/')) == NULL || 
      (p3 = _tcschr(p2+1, '/')) == NULL)
    return(FALSE);
  *p2++ = 1;
  *p3++ = 0;
  if ((FD_Globals.systmFaked.wYear = atoi(p)) <= 0 ||
      (FD_Globals.systmFaked.wMonth = atoi(p2)) <= 0 ||
      (FD_Globals.systmFaked.wDay = atoi(p3)) <= 0)
    return(FALSE);
  free(p);
  return(TRUE);
}


// helper function: check if all essential parameters were supplied
BOOL CheckEssentialParams()
{
  Log(FD_info, "Checking essential parameters...");
  if (FD_Globals.runapp == NULL || *FD_Globals.runapp == 0) {
    Log(FD_error, "No application to run.");
    return(FALSE);
  } else
    Log(FD_info, "...child application: '%s'", FD_Globals.runapp);
  if (FD_Globals.stay != 1 && FD_Globals.timer <= 0) {
    TCHAR s[] = "<unspecified>";
    if (FD_Globals.timer != FD_unspecified)
      _stprintf(s, "%d", FD_Globals.timer);
    Log(FD_error, "Illegal combination of '%s' and '%s' values: "
              "(%s, %s)", FD_stay, FD_timer,
              (FD_Globals.stay) ? "True" : "False", s);
    return(FALSE);
  } else {
    Log(FD_info, "...stay-resident flag: %s",
        (FD_Globals.stay) ? "True" : "False");
    Log(FD_info, "...timer value: %ld seconds", FD_Globals.timer);
  }
  if (!ComputeFakedDate()) {
    Log(FD_error, "Wrong date format: '%s'", FD_Globals.dateArg);
    return(FALSE);
  } else
    Log(FD_info, "...faked date: %s", FD_Globals.dateArg);
  if (FD_Globals.childCleanup < 0)
    FD_Globals.childCleanup = FD_defaultCleanup;
  return(TRUE);
}


// well...
BOOL FakeDate(BOOL saveSystemTime)
{
  SYSTEMTIME systmCurrent;
  GetLocalTime(&systmCurrent);
  FD_Globals.systmFaked.wHour = systmCurrent.wHour;
  FD_Globals.systmFaked.wMinute = systmCurrent.wMinute;
  FD_Globals.systmFaked.wSecond = systmCurrent.wSecond;
  if (saveSystemTime) {
    FD_Globals.systmOrig = systmCurrent;
    Log(FD_info, "Old system time: %02d/%02d/%02d %02d:%02d:%02d GMT",
        FD_Globals.systmOrig.wYear, FD_Globals.systmOrig.wMonth,
        FD_Globals.systmOrig.wDay, FD_Globals.systmOrig.wHour,
        FD_Globals.systmOrig.wMinute, FD_Globals.systmOrig.wSecond);
  }
  if (!SetLocalTime(&FD_Globals.systmFaked)) {
    Log(FD_error, "Call of SetLocalTime() returned error %ld",
        GetLastError());
    return(FALSE);
  } else
    Log(FD_info, "System time set to current time on faked date.");
  return(TRUE);
}


// start child application
BOOL LaunchChildApp()
{
  STARTUPINFO StartupInfo = { sizeof(STARTUPINFO), NULL, NULL, NULL,
                              0, 0, 0, 0, 0, 0, 0,
                              (FD_Globals.cmdShow) ? STARTF_USESHOWWINDOW : 0,
                              FD_Globals.cmdShow,
                              0, NULL, NULL, NULL, NULL };
  PROCESS_INFORMATION ProcInfo;
  TCHAR* fullCmdLine;
  if (FD_Globals.cmdline == NULL)
    fullCmdLine = FD_Globals.runapp;
  else {
    fullCmdLine = new TCHAR[_tcslen(FD_Globals.runapp) +
                            _tcslen(FD_Globals.cmdline) + 2];
    _stprintf(fullCmdLine, "%s %s", FD_Globals.runapp, FD_Globals.cmdline);
  }
  Log(FD_info, "Starting child '%s'...", FD_Globals.runapp);
  Log(FD_info, "...command line string: '%s'", FD_Globals.cmdline);
  if (!CreateProcess(NULL, fullCmdLine,
                     NULL, NULL, FALSE, FALSE, NULL, NULL,
                     &StartupInfo, &ProcInfo)) {
    Log(FD_error, "Unable to start child '%s' -- error %ld.",
        FD_Globals.runapp, GetLastError());
    return(FALSE);
  } else
    Log(FD_info, "...child launched successfully.", FD_Globals.runapp);
  if (FD_Globals.cmdline != NULL)
    delete fullCmdLine;
  FD_Globals.runappHandle = ProcInfo.hProcess;
  FD_Globals.runappThreadID = ProcInfo.dwThreadId;
  return(TRUE);
}


// callback function called by windows (the function is called once for each
//  of the child application's top-level windows when fakedate requests to
//  end the child application)
BOOL CALLBACK EnumChildCallback(HWND hwnd, LPARAM)
{
  PostMessage(hwnd, WM_CLOSE, 0, 0);
  return(TRUE);
}


// end child application (two-step mechanism: first a polite request, then
//  the hard way if necessary)
void KillChildApp()
{
  if (FD_Globals.childCleanup > 0) {
    Log(FD_info, "Asking child application to exit...");
    if (!EnumThreadWindows(FD_Globals.runappThreadID,
        (WNDENUMPROC)EnumChildCallback, 0))
      Log(FD_error, "...unable to determine window handle of child.");
    else {
      int rc = WaitForSingleObject(FD_Globals.runappHandle,
                                   1000*FD_Globals.childCleanup);
      switch(rc) {
        case WAIT_OBJECT_0:
          Log(FD_info, "...child did an exit.");
          return;
        case WAIT_TIMEOUT:
          Log(FD_error, "Child didn't exit within %d seconds.",
              FD_Globals.childCleanup);
          break;
        default:
          Log(FD_error, "Unable to wait for child termination.");
          break;
      }
    }
  }
  // this code is executed only if soft termination of child failed
  // or allowed child cleanup time is zero
  Log(FD_warn, "Terminating child without notifying it.");
  TerminateProcess(FD_Globals.runappHandle, 0);
}


// helper function: convert system time to large integer
//  (which is then stored)
LARGE_INTEGER LargeIntFromSysTime(SYSTEMTIME* systm)
{
  FILETIME filetm;
  LARGE_INTEGER timeval;
  SystemTimeToFileTime(systm, &filetm);
  timeval.LowPart = filetm.dwLowDateTime;
  timeval.HighPart = filetm.dwHighDateTime;
  return(timeval);
}


// helper function: convert large integer to system time
void SysTimeFromLargeInt(SYSTEMTIME* systm, LARGE_INTEGER* timeval)
{
  FILETIME filetm;
  filetm.dwLowDateTime = timeval->LowPart;
  filetm.dwHighDateTime = timeval->HighPart;
  FileTimeToSystemTime(&filetm, systm);
}


// restore original date (the time elapsed since the start of fakedate
//  is added)
void RestoreDate()
{
  Log(FD_info, "Restoring system time...");
  long hrs = (long)FD_Globals.elapsedSeconds / 3600;
  long min = ((long)FD_Globals.elapsedSeconds - hrs*3600) / 60;
  long sec = FD_Globals.elapsedSeconds % 60;
  Log(FD_info, "...elapsed time since start of program: "
               "%ldh %02d:%02ld", hrs, min, sec);
  // add elapsed time since start of fake job to original time
  LARGE_INTEGER timevalOrig = LargeIntFromSysTime(&FD_Globals.systmOrig);
  timevalOrig.QuadPart += (LONGLONG)(1e+7)*FD_Globals.elapsedSeconds;
  // restore system time
  SYSTEMTIME systmNewReal;
  SysTimeFromLargeInt(&systmNewReal, &timevalOrig);
  FD_Globals.systmOrig = systmNewReal;
  if (SetLocalTime(&systmNewReal))
    Log(FD_info, "...system time restored.");
  else
    Log(FD_error, "Call of SetLocalTime() returned error %ld",
        GetLastError());
}


// the main processing loop
BOOL StartFakeDateJob()
{
  // set faked date, but save original system time
  if (!FakeDate(TRUE)) {
    Log(FD_error, "Unable to set faked date!");
    return(FALSE);
  }
  if (!LaunchChildApp()) {
    RestoreDate();
    Log(FD_error, "Launch of child '%s' failed.", FD_Globals.runapp);
    return(FALSE);
  }
  // main processing loop...
  // ...stay resident?...
  if (FD_Globals.stay != 1) {
    // ...no - wait once, then restore date and exit
    Sleep(1000*FD_Globals.timer);
    FD_Globals.elapsedSeconds = FD_Globals.timer;
    Log(FD_info, "One-time waiting period ended.");
    RestoreDate();
    return(TRUE);
  }
  // ...yes -- loop until child application exits
  //    (store elapsed time since start of processing)
  long timer = (FD_Globals.timer > 0) ? FD_Globals.timer : FD_defaultTimer;
  DWORD rc;
  time_t tm;
  do {
    time(&tm);
    rc = WaitForSingleObject(FD_Globals.runappHandle, 1000*timer);
    if (rc == WAIT_TIMEOUT) {
      // re-fake date if a timer value was specified
      FD_Globals.elapsedSeconds += timer;
      if (FD_Globals.timer != 0)
        FakeDate(FALSE);
    }
  } while (rc == WAIT_TIMEOUT);
  time_t tmNew;
  time(&tmNew);
  FD_Globals.elapsedSeconds += (tmNew - tm);
  switch(rc) {
    DWORD exitCode;
    case WAIT_OBJECT_0:
      if (!GetExitCodeProcess(FD_Globals.runappHandle, &exitCode))
        Log(FD_info, "Child exited -- unable to determine exit code");
      else
        Log(FD_info, "Child left with exit code %ld", exitCode);
      RestoreDate();
      return(TRUE);
    case WAIT_FAILED:
      Log(FD_error, "Call to WaitForSingleObject() returned error %ld.",
                GetLastError());
      KillChildApp();
      RestoreDate();
      return(FALSE);
    default:
      Log(FD_error, "Unexpected return code from WaitForSingleObject(): "
                "%ld", rc);
      KillChildApp();
      RestoreDate();
      return(FALSE);
  }
}


INT WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR lpCmdLine, int)
{
  // check if there's another FakeDate running
  if (GetSemaphore() != 0)
    return(0);
  // initialize error logging
  LogInit();
  // look for special string signalling existence of further
  // command line options
  if (_tcsstr(lpCmdLine, FD_optstring) == lpCmdLine)
    ParseCommandLine(lpCmdLine+_tcslen(FD_optstring+1));
  // no such string present...
  else if (*lpCmdLine != 0) {
    // ...but command line arguments were specified, so save the whole
    //    argument string to pass it to the child application later
    FD_Globals.cmdline = new char[_tcslen(lpCmdLine)+1];
    _tcscpy(FD_Globals.cmdline, lpCmdLine);
  }
  ReadPrivateProfile();
  if (!CheckEssentialParams())
    Log(FD_fatal, "Essential parameter(s) missing or invalid!");
  // start main processing loop
  else if (!StartFakeDateJob())
    Log(FD_fatal, "Fake job unsuccessful!");
  // terminate error logging
  LogClose();
  return(0);
}



//////////////////////////////////////////////////////////////////////////////
// This is the end, my friend
//////////////////////////////////////////////////////////////////////////////
